#include <stdbool.h>
#include <stdio.h>
#include <stdint.h>

#include "app_error.h"
#include "app_gpiote.h"
#include "app_button.h"
#include "app_timer.h"
#include "nrf_delay.h"
#include "simple_uart.h"
#include "nrf_gpio.h"
#include "retarget.h"
#include "boards.h"
#include "ble_debug_assert_handler.h"

#define ever								(;;)

#define APP_TIMER_PRESCALER                  0                                         /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_MAX_TIMERS                 4                                         /**< Maximum number of simultaneously created timers. */
#define APP_TIMER_OP_QUEUE_SIZE              5                                         /**< Size of timer operation queues. */

#define APP_GPIOTE_MAX_USERS                 1                                         /**< Maximum number of users of the GPIOTE handler. */

#define BUTTON_DETECTION_DELAY               APP_TIMER_TICKS(10, APP_TIMER_PRESCALER)  /**< Delay from a GPIOTE event until a button is reported as pushed (in number of timer ticks). */

/**
 * @brief Function for error handling, which is called when an error has occurred.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name.
 */
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    // ble_debug_assert_handler(error_code, line_num, p_file_name);

    // On assert, the system can only recover with a reset.
    NVIC_SystemReset();
}

/**
 * @brief Function for handling button events.
 *
 * @param[in]   pin_no   The pin number of the button pressed.
 */
static void button_event_handler(uint8_t pin_no)
{
	switch (pin_no)
    {
        case BUTTON_0:
        	nrf_gpio_pin_clear(LED_1);
        	nrf_gpio_pin_set(LED_0);
            break;

        case BUTTON_1:
            nrf_gpio_pin_clear(LED_0);
            nrf_gpio_pin_set(LED_1);
        	break;

        default:
            APP_ERROR_HANDLER(pin_no);
    }
}

/**
* @brief Function for the Timer initialization.
*
* @details Initializes the timer module.
*/
static void timers_init(void)
{
    // Initialize timer module
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, false);
}

/**
 * @brief Function for initializing the GPIOTE module.
 */
static void gpiote_init(void)
{
    APP_GPIOTE_INIT(APP_GPIOTE_MAX_USERS);
}

/**
 * @brief Function for initializing the button module.
 */
static void buttons_init(void)
{
	uint32_t err_code;

	// Configure the BUTTON_0 and BUTTON_1 as LED buttons and also configure
	// for 'pull up' because the eval board does not have external pull
	// up resistors
	static app_button_cfg_t buttons[] =
	{
			{BUTTON_0, false, NRF_GPIO_PIN_PULLUP, button_event_handler},
			{BUTTON_1, false, NRF_GPIO_PIN_PULLUP, button_event_handler}
	};

	APP_BUTTON_INIT(buttons, sizeof(buttons) / sizeof(buttons[0]), BUTTON_DETECTION_DELAY, false);

	err_code = app_button_enable();
	APP_ERROR_CHECK(err_code);
}

/**
 * @brief Function for application main entry.
 */
int main(void)
{

	retarget_init();
	simple_uart_putstring((const uint8_t *)"\r\n");
	simple_uart_putstring((const uint8_t *)"Started\r\n");

	timers_init();
	simple_uart_putstring((const uint8_t *)"Timers initialized\r\n");

	gpiote_init();
	simple_uart_putstring((const uint8_t *)"GPIOTE initialized\r\n");

	buttons_init();
	simple_uart_putstring((const uint8_t *)"Buttons initialized\r\n");

	// Configure LED-pins as outputs
	nrf_gpio_cfg_output(LED_0);
	nrf_gpio_cfg_output(LED_1);

	// Main loop
	for ever
	{
		// Switch to a low power state until an event is available for the application
	//	power_manage();
	}

}


